-- Project:			maximator-vga-oscilloscope
-- File:				vga_oscilloscope.vhd
-- Version:			1.1 (13.05.2016)
-- Author:			Piotr Rzeszut (http://piotr94.net21.pl)
-- Description:	Implements simple task, to collect samples from ADC1_IN6 (Arduino A5 pin)
--						and display on VGA display v(t) waveform. Sampling is triggered by
--						clicking DEV_CLR button, sampling rate is 500 SPS.

library ieee;
use ieee.std_logic_1164.all; 
use ieee.std_logic_arith.all; 
use ieee.std_logic_signed.all;
use ieee.numeric_std.all;

entity vga_oscilloscope is
	port(
		clk			: in	std_logic;
		trigger		: in 	std_logic;
		VGA_R			: out std_logic;
		VGA_G			: out std_logic;
		VGA_B			: out std_logic;
		VGA_HS		: buffer std_logic;
		VGA_VS		: buffer std_logic
	);
end vga_oscilloscope;

architecture arch of vga_oscilloscope is
	 
	 component pll is
		PORT
		(
			inclk0		: in  std_logic := '0';--in 10MHz
			c0				: out std_logic ;--10MHz
			c1				: out std_logic ;--50MHz
			c2				: out std_logic ;--65MHz
			c3				: out std_logic ;--130MHz
			locked		: out std_logic 
		);
	end component pll;
	
	component ADC is
        port (
            pll_clk_clk            : in  std_logic                     := 'X';             -- clk
            pll_locked_export      : in  std_logic                     := 'X';             -- export
            response_valid         : out std_logic;                                        -- valid
            response_channel       : out std_logic_vector(4 downto 0);                     -- channel
            response_data          : out std_logic_vector(11 downto 0);                    -- data
            response_startofpacket : out std_logic;                                        -- startofpacket
            response_endofpacket   : out std_logic;                                        -- endofpacket
            clk_clk                : in  std_logic                     := 'X';             -- clk
            rst_reset_n            : in  std_logic                     := 'X';             -- reset_n
            command_valid          : in  std_logic                     := 'X';             -- valid
            command_channel        : in  std_logic_vector(4 downto 0)  := (others => 'X'); -- channel
            command_startofpacket  : in  std_logic                     := 'X';             -- startofpacket
            command_endofpacket    : in  std_logic                     := 'X';             -- endofpacket
            command_ready          : out std_logic                                         -- ready
        );
   end component ADC;
	
	component vga_driver is
		generic(
			PX_WIDTH				: integer := 800;
			PX_FRONT_PORCH		: integer := 40;
			PX_SYNC_PULSE		: integer := 128;
			PX_BACK_PORCH		: integer := 88;
			LINE_HEIGHT			: integer := 600;
			LINE_FRONT_PORCH	: integer := 1;
			LINE_SYNC_PULSE	: integer := 4;
			LINE_BACK_PORCH	: integer := 23
		);
		port(
			vga_r			: out std_logic;
			vga_g			: out std_logic;
			vga_b			: out std_logic;
			vga_in		: in std_logic_vector(2 downto 0);
			vga_hs_n		: out std_logic;
			vga_vs_n		: out std_logic;
			px_addr		: buffer std_logic_vector(20 downto 0);
			clk			: in std_logic;
			rst_n			: in std_logic
		);
	end component vga_driver;
	
	component line_drawer is 
		port(
			clk			: in	std_logic;
			rst_n			: in	std_logic;
			act_value	: in	std_logic_vector(10 downto 0);
			offset		: in	std_logic_vector(10 downto 0);
			bg_color		: in	std_logic_vector(2 downto 0);
			fg_color	: in	std_logic_vector(2 downto 0);
			px_addr		: in	std_logic_vector(10 downto 0);
			px_data		: out	std_logic_vector(2 downto 0);
			px_req		: out	std_logic
		);
	end component line_drawer;
	
	component ram_display is
		port
		(
			data				: in std_logic_vector (8 downto 0);
			rdaddress		: in std_logic_vector (9 downto 0);
			rdclock			: in std_logic ;
			wraddress		: in std_logic_vector (9 downto 0);
			wrclock			: in std_logic  := '1';
			wren				: in std_logic  := '0';
			q					: out std_logic_vector (8 downto 0)
		);
	end component ram_display;
	
	component adc_command is
		port(
			clk							: in  std_logic;
			response_valid				: in  std_logic;                                       
			response_channel			: in  std_logic_vector(4 downto 0);                    
			response_data				: in  std_logic_vector(11 downto 0);                   
			response_startofpacket	: in  std_logic;                                     
			response_endofpacket		: in  std_logic;
			command_channel 			: out std_logic_vector(4 downto 0);
			command_valid 				: out std_logic;
			command_startofpacket 	: out std_logic;
			command_endofpacket 		: out std_logic;
			command_ready 				: out std_logic;
			valid_data					: out std_logic_vector(11 downto 0)
		);
	end component adc_command;
	
	component trigger_simple is
		port(
			clk					: in  std_logic;
			trigger				: in  std_logic;
			wraddress 			: out std_logic_vector(9 downto 0);
			wr_en					: out std_logic
		);
	end component trigger_simple;
		
	
	signal clk_adc		:std_logic;
	signal clk_50M		:std_logic;
	signal pll_locked	:std_logic;
	signal vga_clk 	:std_logic;
	
	signal act_value 	:std_logic_vector(10 downto 0);
	
	signal valid_data :std_logic_vector(11 downto 0);
	signal wraddress 	:std_logic_vector(9 downto 0);
	signal wr_en 		:std_logic;
	signal offset 		:std_logic_vector(9 downto 0);
	signal px_data 	:std_logic_vector(2 downto 0);
	signal px_addr 	:std_logic_vector(20 downto 0);
	
	signal reset_n						:std_logic;
	
	signal command_valid 			:std_logic;
	signal command_channel 			:std_logic_vector(4 downto 0);
	signal command_startofpacket 	:std_logic;
	signal command_endofpacket 	:std_logic;
	signal command_ready 			:std_logic;
	signal response_valid 			:std_logic;
	signal response_channel 		:std_logic_vector(4 downto 0);
	signal response_data 			:std_logic_vector(11 downto 0);
	signal response_startofpacket :std_logic;
	signal response_endofpacket 	:std_logic;

begin

	reset_n <= '1';
	

	pll1: component pll
		port map (
			inclk0 	=> clk, 
			c0 		=> clk_adc, 
			c1 		=> clk_50M, 
			c2 		=> vga_clk, 
			c3 		=> open, 
			locked 	=> pll_locked
		);
		
	u0 : component ADC
	  port map (
			pll_clk_clk            => clk_adc,            		--    pll_clk.clk
			pll_locked_export      => pll_locked,      			-- pll_locked.export
			response_valid         => response_valid,         	--   response.valid
			response_channel       => response_channel,       	--           .channel
			response_data          => response_data,          	--           .data
			response_startofpacket => response_startofpacket, 	--           .startofpacket
			response_endofpacket   => response_endofpacket,   	--           .endofpacket
			clk_clk                => clk,                		--        clk.clk
			rst_reset_n            => reset_n,            		--        rst.reset_n
			command_valid          => command_valid,          	--    command.valid
			command_channel        => command_channel,        	--           .channel
			command_startofpacket  => command_startofpacket,  	--           .startofpacket
			command_endofpacket    => command_endofpacket,    	--           .endofpacket
			command_ready          => command_ready           	--           .ready
	  );
	
	adc_command1 : component adc_command
		port map (
			clk							=> clk,
			response_valid         	=> response_valid,         	
			response_channel       	=> response_channel,       	
			response_data          	=> response_data,          	
			response_startofpacket 	=> response_startofpacket, 	
			response_endofpacket   	=> response_endofpacket,
			command_channel 			=> command_channel,
			command_valid 				=> command_valid,
			command_startofpacket 	=> command_startofpacket,
			command_endofpacket 		=> command_endofpacket,
			command_ready 				=> command_ready,
			valid_data					=> valid_data
		);
		  
	
	VGA1 : vga_driver
		generic map(
			PX_WIDTH				=> 1024,
			PX_FRONT_PORCH		=> 24,
			PX_SYNC_PULSE		=> 136,
			PX_BACK_PORCH		=> 160,
			LINE_HEIGHT			=> 768,
			LINE_FRONT_PORCH	=> 3,
			LINE_SYNC_PULSE	=> 6,
			LINE_BACK_PORCH	=> 29
		)
		port map(
			vga_r			=> vga_r,
			vga_g			=> vga_g,
			vga_b			=> vga_b,
			vga_in		=> px_data,
			vga_hs_n		=> vga_hs,
			vga_vs_n		=> vga_vs,
			px_addr		=> px_addr,
			clk			=> vga_clk,
			rst_n			=> reset_n
		);
	
	LINE1 : line_drawer
		port map(
			clk			=> vga_clk,
			rst_n			=> reset_n,
			act_value	=> act_value,
			offset		=> std_logic_vector(to_unsigned(128,11)),
			bg_color		=> (others => '0'),
			fg_color		=> (others => '1'),
			px_addr		=> px_addr(20 downto 10),
			px_data		=> px_data,
			px_req		=> open
		);
	
	RAM_DISP : RAM_DISPLAY
		port map(
			data			=> valid_data(11 downto 3),
			rdaddress	=> px_addr(9 downto 0),
			rdclock		=> vga_clk,
			wraddress	=> wraddress,
			wrclock		=> clk_50M,
			wren			=> wr_en,
			q				=> act_value(8 downto 0)
		);
	
	trigger_module : trigger_simple
		port map(
			clk			=> clk_50M,
			trigger		=> trigger,
			wraddress 	=> wraddress,
			wr_en			=> wr_en
		);
	
end arch;
